/*
 * Symphony - A modern community (forum/BBS/SNS/blog) platform written in Java.
 * Copyright (C) 2012-present, b3log.org
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */
package org.lym.pom.util;

import com.google.common.base.CharMatcher;
import org.apache.commons.lang3.StringUtils;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Rune utilities.
 *
 * @author <a href="http://88250.b3log.org">Liang Ding</a>
 * @version 1.1.0.0, Mar 31, 2019
 * @since 2.1.0
 */
public final class Runes {


    private Runes() {
    }

//    public static void main(final String[] args) {
//        System.out.println(getChinesePercent("123abc这个中文cde123abc也要提取123ab"));
//    }

    /**
     * Removes control characters for the specified string.
     *removeControlChars(String str)：该方法用于移除给定字符串中的控制字符（包括ASCII控制字符和其他不可见字符，但保留回车、换行和制表符）。返回一个去除控制字符后的字符串
     * @param str the specified string
     * @return str removed control characters
     */
    public static String removeControlChars(final String str) {
        final CharMatcher charsToPreserve = CharMatcher.anyOf("\r\n\t");
        final CharMatcher allButPreserved = charsToPreserve.negate();
        final CharMatcher controlCharactersToRemove = CharMatcher.javaIsoControl().and(allButPreserved);

        return controlCharactersToRemove.removeFrom(str);
    }

    /**
     * Gets chinese percentage of the specified string.
     *该方法计算并返回指定字符串中中文字符所占的百分比。它通过正则表达式匹配所有中文字符（Unicode范围[\u4e00-\u9fa5]），然后统计这些中文字符的数量，并将该数量除以整个字符串长度再乘以100得到百分比。如果输入字符串为空或空白，则返回0。
     * @param str the specified string
     * @return percentage
     */
    public static int getChinesePercent(final String str) {
        if (StringUtils.isBlank(str)) {
            return 0;
        }

        final Pattern p = Pattern.compile("([\u4e00-\u9fa5]+)");
        final Matcher m = p.matcher(str);
        final StringBuilder chineseBuilder = new StringBuilder();
        while (m.find()) {
            chineseBuilder.append(m.group(0));
        }

        return (int) Math.floor(StringUtils.length(chineseBuilder.toString()) / (double) StringUtils.length(str) * 100);
    }
}
